%global default_type glibc
%global kernel_headers_version 4.19.88-2

%ifarch x86_64 aarch64
%global hcflags %{_hardening_cflags} -fstack-clash-protection
%global hldflags %{_hardening_ldflags} -Wl,-z,relro,-z,now
%endif

Summary:	Statically linked binary providing simplified versions of system commands
Name:		busybox
Version:	1.36.1
Release:	7%{?dist}
License:	GPLv2
URL:		http://www.busybox.net
Source0:	http://www.busybox.net/downloads/%{name}-%{version}.tar.bz2
Source1:	https://github.com/sabotage-linux/kernel-headers/archive/refs/tags/v%{kernel_headers_version}.tar.gz
Source2:	busybox-glibc-static.config
Source3:	busybox-petitboot.config
Source4:	busybox-shared.config

#https://isrc.iscas.ac.cn/gitlab/mirrors/github.com/openembedded_openembedded-core/-/commit/0b9cbcc4ceac3938afd1dd6010ce6d9a3da21598
Patch0001:  fix-CVE-2022-28391.patch
#https://git.openembedded.org/openembedded-core/tree/meta/recipes-core/busybox/busybox/CVE-2022-48174.patch?h=kirkstone
Patch0002:  fix-CVE-2022-48174.patch
Patch0003:  fix-CVE-2023-42363.patch
#https://lists.busybox.net/pipermail/busybox/2024-May/090762.html
#https://git.busybox.net/busybox/commit/editors/awk.c?id=0256e00a9d077588bd3a39f5a1ef7e2eaa2911e4
Patch0004:  fix-CVE-2023-42364-and-CVE-2023-42365.patch
#https://git.alpinelinux.org/aports/plain/main/busybox/0026-awk.c-fix-CVE-2023-42366-bug-15874.patch
Patch0005:  fix-CVE-2023-42366.patch
#https://git.busybox.net/busybox/commit/?id=9a8796436b9b0641e13480811902ea2ac57881d3
Patch0006: CVE-2023-39810-archival-disallow-path-traversals.patch

Patch3000:	busybox-1.31.1-stime-fix.patch

BuildRequires:	gcc make glibc-static
BuildRequires:	libselinux-devel >= 1.27.7-2 libselinux-static libsepol-devel libsepol-static

Provides:	bundled(md5-drepper2)

%description
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. This package can be very
useful for recovering from certain types of system failures,
particularly those involving broken shared libraries.

%package petitboot
Summary:	Version of busybox configured for use with petitboot

%description petitboot
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. The version contained in this
package is a minimal configuration intended for use with the Petitboot
bootloader used on PlayStation 3. The busybox package provides a binary
better suited to normal use.

%package shared
Summary:	A shared (non-static) version of busybox

%description shared
Busybox is a single binary which includes versions of a large number
of system commands, including a shell. The version contained in this
package is build against shared libraries, most notably glibc.

%prep
%autosetup -a 1 -p1

%build
cp %{SOURCE2} .config
yes "" | make oldconfig && \
make V=1 \
EXTRA_CFLAGS="%{?hcflags} -g" \
LDFLAGS+="%{?hldflags}"

cp busybox_unstripped busybox.glibc.static
cp docs/busybox.1 docs/busybox.glibc.static.1

make clean

cp %{SOURCE3} .config
yes "" | make oldconfig
make V=1 \
EXTRA_CFLAGS="-g %{?hcflags}" \
LDFLAGS+="%{?hldflags}"

cp busybox_unstripped busybox.glibc.petitboot
cp docs/busybox.1 docs/busybox.glibc.petitboot.1

make clean

cp %{SOURCE4} .config
yes "" | make oldconfig
make V=1 EXTRA_CFLAGS="%{optflags}" CFLAGS_busybox="%{build_ldflags}"
cp busybox_unstripped busybox.shared
cp docs/busybox.1 docs/busybox.shared.1

%install
mkdir -p %{buildroot}%{_sbindir}
install -m 755 busybox.*.static %{buildroot}%{_sbindir}
mv %{buildroot}%{_sbindir}/busybox.%{default_type}.static %{buildroot}%{_sbindir}/busybox
ln -s ./busybox %{buildroot}%{_sbindir}/busybox.%{default_type}.static
install -m 755 busybox.*.petitboot %{buildroot}%{_sbindir}
mv %{buildroot}%{_sbindir}/busybox.%{default_type}.petitboot %{buildroot}%{_sbindir}/busybox.petitboot
ln -s ./busybox.petitboot %{buildroot}%{_sbindir}/busybox.%{default_type}.petitboot
install -m 755 busybox.shared %{buildroot}%{_sbindir}/busybox.shared
mkdir -p %{buildroot}%{_mandir}/man1
install -m 644 docs/busybox.*.static.1 %{buildroot}%{_mandir}/man1/
mv %{buildroot}%{_mandir}/man1/busybox.%{default_type}.static.1 %{buildroot}%{_mandir}/man1/busybox.static.1
ln -s ./busybox.static.1 %{buildroot}%{_mandir}/man1/busybox.%{default_type}.static.1
install -m 644 docs/busybox.*.petitboot.1 %{buildroot}%{_mandir}/man1/
mv %{buildroot}%{_mandir}/man1/busybox.%{default_type}.petitboot.1 %{buildroot}%{_mandir}/man1/busybox.petitboot.1
ln -s ./busybox.petitboot.1 %{buildroot}%{_mandir}/man1/busybox.%{default_type}.petitboot.1
install -m 644 docs/busybox.shared.1 %{buildroot}%{_mandir}/man1/busybox.shared.1

%files
%license LICENSE
%doc README
%{_sbindir}/busybox
%{_sbindir}/busybox*.static
%{_mandir}/man1/busybox*.static.1.gz

%files petitboot
%license LICENSE
%doc README
%{_sbindir}/busybox*.petitboot
%{_mandir}/man1/busybox*.petitboot.1.gz

%files shared
%license LICENSE
%doc README
%{_sbindir}/busybox.shared
%{_mandir}/man1/busybox.shared.1.gz

%changelog
* Sun Apr 27 2025 Chunsheng Luo <luffyluo@tencent.com> - 1.36.1-7
- archival: disallow path traversals (CVE-2023-39810)

* Fri Nov 22 2024 Feng Weiyao <wynnfeng@tencent.com> - 1.36.1-6
- fix CVE-2022-28391 CVE-2022-48174 CVE-2023-42364 CVE-2023-42365 CVE-2023-42366

* Fri Nov 15 2024 Feng Weiyao <wynnfeng@tencent.com> - 1.36.1-5
- fix CVE-2023-42363

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.36.1-4
- Rebuilt for loongarch release

* Fri Aug 09 2024 kianli <kianli@tencent.com> - 1.36.1-3
- Fixed static compilation not taking effect

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.36.1-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Aug 09 2023 jackeyji <jackeyji@tencent.com> - 1.36.1-1
- upgrade to 1.36.1

* Fri May 05 2023 cunshunxia <cunshunxia@tencent.com> - 1.36.0-3
- remove unused source files.

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.36.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Mon Apr 10 2023 cunshunxia <cunshunxia@tencent.com> - 1.36.0-1
- initial build
